from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from app.api.deps import get_db, get_current_active_user, get_current_admin_user
from app.crud.crud_category import category
from app.models.user import User as UserModel
from app.schemas.category import Category, CategoryCreate, CategoryUpdate

router = APIRouter()


@router.get("/", response_model=List[Category])
def read_categories(
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: UserModel = Depends(get_current_active_user),
) -> Any:
    """
    Retrieve categories.
    """
    categories = category.get_multi(db, skip=skip, limit=limit)
    return categories


@router.post("/", response_model=Category)
def create_category(
    *,
    db: Session = Depends(get_db),
    category_in: CategoryCreate,
    current_user: UserModel = Depends(get_current_admin_user),
) -> Any:
    """
    Create new category. Admin only.
    """
    db_category = category.get_by_name(db, name=category_in.name)
    if db_category:
        raise HTTPException(
            status_code=400,
            detail="A category with this name already exists",
        )
    new_category = category.create(db, obj_in=category_in)
    return new_category


@router.get("/{id}", response_model=Category)
def read_category(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: UserModel = Depends(get_current_active_user),
) -> Any:
    """
    Get category by ID.
    """
    db_category = category.get(db, id=id)
    if not db_category:
        raise HTTPException(status_code=404, detail="Category not found")
    return db_category


@router.put("/{id}", response_model=Category)
def update_category(
    *,
    db: Session = Depends(get_db),
    id: int,
    category_in: CategoryUpdate,
    current_user: UserModel = Depends(get_current_admin_user),
) -> Any:
    """
    Update a category. Admin only.
    """
    db_category = category.get(db, id=id)
    if not db_category:
        raise HTTPException(status_code=404, detail="Category not found")
    
    # Check if updated name conflicts with existing category
    if category_in.name and category_in.name != db_category.name:
        existing = category.get_by_name(db, name=category_in.name)
        if existing:
            raise HTTPException(
                status_code=400,
                detail="A category with this name already exists",
            )
    
    updated_category = category.update(db, db_obj=db_category, obj_in=category_in)
    return updated_category


@router.delete("/{id}", response_model=Category)
def delete_category(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: UserModel = Depends(get_current_admin_user),
) -> Any:
    """
    Delete a category. Admin only.
    """
    db_category = category.get(db, id=id)
    if not db_category:
        raise HTTPException(status_code=404, detail="Category not found")
    
    # TODO: Check if category has products, and prevent deletion if it does
    
    deleted_category = category.remove(db, id=id)
    return deleted_category