from typing import Any, List, Optional

from fastapi import APIRouter, Depends, HTTPException, Query
from sqlalchemy.orm import Session

from app.api.deps import get_db, get_current_active_user, get_current_admin_user
from app.crud.crud_product import product
from app.models.user import User as UserModel
from app.schemas.product import Product, ProductCreate, ProductUpdate, ProductWithInventory

router = APIRouter()


@router.get("/", response_model=List[ProductWithInventory])
def read_products(
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
    category_id: Optional[int] = None,
    current_user: UserModel = Depends(get_current_active_user),
) -> Any:
    """
    Retrieve products with inventory information.
    """
    products = product.get_multi_with_inventory(db, skip=skip, limit=limit)
    
    # Filter by category if provided
    if category_id is not None:
        products = [p for p in products if p["category_id"] == category_id]
        
    return products


@router.post("/", response_model=Product)
def create_product(
    *,
    db: Session = Depends(get_db),
    product_in: ProductCreate,
    current_user: UserModel = Depends(get_current_admin_user),
) -> Any:
    """
    Create new product. Admin only.
    """
    # Check for duplicate SKU
    if product_in.sku:
        db_product = product.get_by_sku(db, sku=product_in.sku)
        if db_product:
            raise HTTPException(
                status_code=400,
                detail="A product with this SKU already exists",
            )
    
    # Check for duplicate barcode
    if product_in.barcode:
        db_product = product.get_by_barcode(db, barcode=product_in.barcode)
        if db_product:
            raise HTTPException(
                status_code=400,
                detail="A product with this barcode already exists",
            )
    
    new_product = product.create(db, obj_in=product_in)
    return new_product


@router.get("/{id}", response_model=ProductWithInventory)
def read_product(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: UserModel = Depends(get_current_active_user),
) -> Any:
    """
    Get product by ID with inventory information.
    """
    db_product = product.get_with_inventory(db, id=id)
    if not db_product:
        raise HTTPException(status_code=404, detail="Product not found")
    return db_product


@router.put("/{id}", response_model=Product)
def update_product(
    *,
    db: Session = Depends(get_db),
    id: int,
    product_in: ProductUpdate,
    current_user: UserModel = Depends(get_current_admin_user),
) -> Any:
    """
    Update a product. Admin only.
    """
    db_product = product.get(db, id=id)
    if not db_product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    # Check for duplicate SKU
    if product_in.sku and product_in.sku != db_product.sku:
        existing = product.get_by_sku(db, sku=product_in.sku)
        if existing:
            raise HTTPException(
                status_code=400,
                detail="A product with this SKU already exists",
            )
    
    # Check for duplicate barcode
    if product_in.barcode and product_in.barcode != db_product.barcode:
        existing = product.get_by_barcode(db, barcode=product_in.barcode)
        if existing:
            raise HTTPException(
                status_code=400,
                detail="A product with this barcode already exists",
            )
    
    updated_product = product.update(db, db_obj=db_product, obj_in=product_in)
    return updated_product


@router.delete("/{id}", response_model=Product)
def delete_product(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: UserModel = Depends(get_current_admin_user),
) -> Any:
    """
    Delete a product. Admin only.
    """
    db_product = product.get(db, id=id)
    if not db_product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    # TODO: Consider checking if product has inventory or is referenced by other entities
    
    deleted_product = product.remove(db, id=id)
    return deleted_product


@router.get("/by-sku/{sku}", response_model=ProductWithInventory)
def read_product_by_sku(
    *,
    db: Session = Depends(get_db),
    sku: str,
    current_user: UserModel = Depends(get_current_active_user),
) -> Any:
    """
    Get product by SKU with inventory information.
    """
    db_product = product.get_by_sku(db, sku=sku)
    if not db_product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    return product.get_with_inventory(db, id=db_product.id)


@router.get("/by-barcode/{barcode}", response_model=ProductWithInventory)
def read_product_by_barcode(
    *,
    db: Session = Depends(get_db),
    barcode: str,
    current_user: UserModel = Depends(get_current_active_user),
) -> Any:
    """
    Get product by barcode with inventory information.
    """
    db_product = product.get_by_barcode(db, barcode=barcode)
    if not db_product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    return product.get_with_inventory(db, id=db_product.id)